@@ -3,15 +3,21 @@
 """Tools to generate a Snakemake-based BIDS app."""
 
 import os
+import pathlib
 import subprocess
 import argparse
 import logging
 import sys
 import yaml
 import bids
-from snakemake import get_argument_parser
+import snakemake
 from snakemake.io import load_configfile
 
+# We define Path here in addition to pathlib to put both variables in globals()
+# This way, users specifying a path type in their config.yaml can indicate
+# either Path or pathlib.Path
+Path = pathlib.Path
+
 bids.config.set_option("extension_initial_dot", True)
 logger = logging.Logger(__name__)
 
@@ -88,7 +94,29 @@ def get_time_hash():
     hash.update(str(time.time()).encode('utf-8'))
     return hash.hexdigest()[:8]
 
+def resolve_path(path_candidate):
+    """Helper function to resolve any paths or list
+    of paths it's passed. Otherwise, returns the argument 
+    unchanged. 
 
+    Parameters
+    ----------
+    command : list, os.Pathlike, object
+        command to run
+
+    Returns
+    -------
+    list, os.Pathlike, object
+        If os.Pathlike or list  of os.Pathlike, the same paths resolved.
+        Otherwise, the argument unchanged.
+    """
+    if isinstance(path_candidate, list):
+        return [resolve_path(p) for p in path_candidate]
+
+    if isinstance(path_candidate, os.PathLike):
+        return path_candidate.resolve()
+    
+    return path_candidate
 
 SNAKEFILE_CHOICES = [
     "Snakefile",
@@ -156,32 +184,27 @@ class SnakeBidsApp:
         # look for snakebids.yml in the snakemake_dir, quit if not found
         self.configfile_path = None
         for path in CONFIGFILE_CHOICES:
-            if os.path.exists(os.path.join(snakemake_dir, path)):
+            if Path(snakemake_dir, path).exists():
                 self.configfile_path = path
                 break
         if self.configfile_path is None:
             raise ConfigError(
-                "Error: no config file found, tried {}.".format(
-                    ", ".join(CONFIGFILE_CHOICES)
-                )
+                f"Error: no config file found, tried {', '.join(CONFIGFILE_CHOICES)}."
             )
 
         # look for snakefile in the snakemake_dir, quit if not found
         self.snakefile = None
         for snakefile_path in SNAKEFILE_CHOICES:
-            if os.path.exists(os.path.join(snakemake_dir, snakefile_path)):
-                self.snakefile = os.path.join(snakemake_dir, snakefile_path)
+            if Path(snakemake_dir, snakefile_path).exists():
+                self.snakefile = Path(snakemake_dir, snakefile_path)
                 break
         if self.snakefile is None:
             raise ConfigError(
-                "Error: no Snakefile found, tried {}.".format(
-                    ", ".join(SNAKEFILE_CHOICES)
-                )
+                f"Error: no Snakefile found, tried {', '.join(SNAKEFILE_CHOICES)}."
             )
 
-        self.config = load_configfile(os.path.join(
-                                        snakemake_dir,
-                                        self.configfile_path))
+        self.config = load_configfile(Path(snakemake_dir,
+                                            self.configfile_path))
 
         if self.config.get("debug", False):
             logging.basicConfig(level=logging.DEBUG)
@@ -209,7 +232,7 @@ class SnakeBidsApp:
 
         if include_snakemake:
             # get snakemake parser
-            smk_parser = get_argument_parser()
+            smk_parser = snakemake.get_argument_parser()
 
             # create parser
             parser = argparse.ArgumentParser(
@@ -240,6 +263,19 @@ class SnakeBidsApp:
 
         # update the parser with config options
         for name, parse_args in self.config["parse_args"].items():
+            # Convert type annotations from strings to class types
+            # We first check that the type annotation is, in fact,
+            # a str to allow the edge case where it's already
+            # been converted
+            if "type" in parse_args and isinstance(parse_args["type"], str):
+                try:
+                    parse_args["type"] = globals()[parse_args["type"]]
+                except KeyError as err:
+                    raise TypeError(
+                        f"{parse_args['type']} is not available "
+                        + f"as a type for {name}"
+                    ) from err
+                    
             app_group.add_argument(name, **parse_args)
 
         # general parser for
@@ -313,6 +349,10 @@ class SnakeBidsApp:
 
         args = all_args[0]
         snakemake_args = all_args[1]
+        # resolve all path items to get absolute paths
+        args.__dict__ = {
+            k: resolve_path(v) for k, v in args.__dict__.items()
+        }
 
         # add snakebids arguments to config
         self.config.update(args.__dict__)
@@ -348,26 +388,23 @@ class SnakeBidsApp:
             if custom_path is not None:
                 self.config["pybids_inputs"][input_type][
                     "custom_path"
-                ] = os.path.realpath(custom_path)
+                ] = Path(custom_path).resolve()
             del self.config[f"path_{input_type}"]
 
         # replace paths with realpaths
-        self.config["bids_dir"] = os.path.realpath(self.config["bids_dir"])
-        self.config["output_dir"] = os.path.realpath(self.config["output_dir"])
-
+        self.config["bids_dir"] = Path(self.config["bids_dir"]).resolve()
+        self.config["output_dir"] = Path(self.config["output_dir"]).resolve()
 
+    
     def write_updated_config(self):
         """Create an updated snakebids config file in the output dir."""
-        self.updated_config = os.path.join(
-                                    self.config["output_dir"],
+        self.updated_config = Path(self.config["output_dir"],
                                     self.configfile_path)
 
 
         # create the output folder if needed
-        os.makedirs(os.path.dirname(self.updated_config), exist_ok=True)
+        self.updated_config.parent.mkdir(parents = True, exist_ok=True)
 
-        # write either as JSON or YAML
-        config_file_ext = os.path.splitext(self.updated_config)
 
         time_hash = get_time_hash() # TODO: copy to a time-hashed file too
                                     # for provenance? 
@@ -375,7 +412,8 @@ class SnakeBidsApp:
 
 
         with open(self.updated_config, "w") as f:
-            if config_file_ext[-1] == '.json':
+            # write either as JSON or YAML
+            if self.updated_config.suffix == '.json':
                 import json
                 json.dump(self.config, f, indent=4)
             else: #if not json, then should be yaml or yml
@@ -387,6 +425,12 @@ class SnakeBidsApp:
                                             dumper.represent_mapping(
                                                 'tag:yaml.org,2002:map',
                                                 data.items()))
+
+                # Represent any PathLikes as str.
+                path2str = lambda dumper, data: dumper.represent_scalar('tag:yaml.org,2002:str',str(data))
+                yaml.add_representer(pathlib.PosixPath, path2str)
+                yaml.add_representer(pathlib.WindowsPath, path2str)
+
                 yaml.dump(dict(self.config),
                             f, 
                             default_flow_style=False,
